// Copyright (C) 2000, Luca Padovani <luca.padovani@cs.unibo.it>.
// 
// This file is part of GtkMathView, a Gtk widget for MathML.
// 
// GtkMathView is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// GtkMathView is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with GtkMathView; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
// 
// For details, see the GtkMathView World-Wide-Web page,
// http://cs.unibo.it/~lpadovan/mml-widget, or send a mail to
// <luca.padovani@cs.unibo.it>

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <stdio.h>

#include "Rectangle.hh"
#include "BoundingBox.hh"

void
BoundingBox::Null()
{
  null    = true;
  width   = 0;
  ascent  = 0;
  descent = 0;
}

void
BoundingBox::Set(scaled w, scaled a, scaled d)
{
  null    = false;
  width   = w;
  ascent  = a;
  descent = d;
}

void
BoundingBox::Append(const BoundingBox& box)
{  
  if (box.IsNull()) return;

  width += box.width;

  if (null) {
    ascent  = box.ascent;
    descent = box.descent;
    null = false;
  } else {
    ascent  = scaledMax(ascent, box.ascent);
    descent = scaledMax(descent, box.descent);
  }
}

void
BoundingBox::Append(scaled w)
{
  width += w;
}

void
BoundingBox::ToRectangle(scaled x, scaled y, Rectangle& rect) const
{
  rect.x = x;
  rect.y = y - ascent;
  rect.width = width;
  rect.height = GetHeight();
}

void
BoundingBox::Dump() const
{
  if (IsNull()) printf("[null box]");
  else printf("[%d,+%d,-%d]", sp2ipx(width), sp2ipx(ascent), sp2ipx(descent));
}

