// Copyright (C) 2000, Luca Padovani <luca.padovani@cs.unibo.it>.
// 
// This file is part of GtkMathView, a Gtk widget for MathML.
// 
// GtkMathView is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// GtkMathView is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with GtkMathView; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
// 
// For details, see the GtkMathView World-Wide-Web page,
// http://cs.unibo.it/~lpadovan/mml-widget, or send a mail to
// <luca.padovani@cs.unibo.it>

#ifndef CharMapper_hh
#define CharMapper_hh

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <parser.h>

#include "CharMap.hh"
#include "FontAttributes.hh"

#define CHAR_MAP_HASH_BITS       4
#define CHAR_MAP_HASH_TABLE_SIZE (1 << CHAR_MAP_HASH_BITS)
#define CHAR_MAP_HASH_MASK       (CHAR_MAP_HASH_TABLE_SIZE - 1)
#define CHAR_HASH(ch)            ((ch) & CHAR_MAP_HASH_MASK)

class CharMapper {
public:
  CharMapper(void);
  ~CharMapper();

  bool      Load(const char*);
  StretchId GetStretch(Char) const;
  FontifiedChar* FontifyChar(const FontAttributes&, Char, bool = false) const;

private:
  struct FontMap {
    FontAttributes      attributes;
    Container<CharMap*> multi; // actually, RANGE and MULTI maps
    Container<CharMap*> single[CHAR_MAP_HASH_TABLE_SIZE]; // actually, SINGLE and STRETCHY

    StretchId GetStretch(Char) const;
    FontifiedChar* FontifyChar(FontAttributeMask, const FontAttributes&, Char, bool = false) const;
  };

  void ParseFontMapList(xmlNodePtr);
  void ParseFontMap(xmlNodePtr);
  void ParseCharMap(xmlNodePtr, FontMap*);
  void ParseRange(xmlNodePtr, FontMap*);
  void ParseMulti(xmlNodePtr, FontMap*);
  void ParseSingle(xmlNodePtr, FontMap*);
  void ParseStretchy(xmlNodePtr, FontMap*);
  void ParseStretchySimple(xmlNodePtr, CharMap*);
  void ParseStretchyCompound(xmlNodePtr, CharMap*);

  Container<FontMap*> content;
};

#endif
 
