// Copyright (C) 2000, Luca Padovani <luca.padovani@cs.unibo.it>.
// 
// This file is part of GtkMathView, a Gtk widget for MathML.
// 
// GtkMathView is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// GtkMathView is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with GtkMathView; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
// 
// For details, see the GtkMathView World-Wide-Web page,
// http://cs.unibo.it/~lpadovan/mml-widget, or send a mail to
// <luca.padovani@cs.unibo.it>

#ifndef DrawingArea_hh
#define DrawingArea_hh

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <gtk/gtk.h>

#include "Font.hh"
#include "Shape.hh"
#include "String.hh"
#include "BoundingBox.hh"
#include "GraphicsContext.hh"

class DrawingArea
{
public:
  DrawingArea(GtkWidget*, GdkPixmap*);
  ~DrawingArea();

  GraphicsContext* CreateGraphicsContext(void) const;
  const GraphicsContext* GetSelectedGraphicsContext(void) const { 
    return selectedGC;
  }

  void SetTop(scaled, scaled);

  void MoveTo(scaled, scaled) const;
  void DrawLine(const GraphicsContext*, scaled, scaled, scaled, scaled) const;
  void DrawLineTo(const GraphicsContext*, scaled, scaled) const;
  void DrawLineToDelta(const GraphicsContext*, scaled, scaled) const;
  void DrawRectangle(const GraphicsContext*, scaled, scaled, scaled, scaled) const;
  void DrawRectangle(const GraphicsContext*, scaled, scaled, const BoundingBox&) const;
  void DrawRectangle(const GraphicsContext*, const Rectangle&) const;
  void DrawRectangle(const GraphicsContext*, const Shape&) const;
  void DrawBoundingBox(const GraphicsContext*, scaled, scaled, const BoundingBox&, bool = true) const;
  void DrawNativeChar(const GraphicsContext*, const Font*, scaled, scaled, NativeChar) const;
  void DrawString(const GraphicsContext*, const Font*, scaled, scaled, const String*) const;
  void DrawString(const GraphicsContext*, const Font*, scaled, scaled, const char*, unsigned) const;
  void DrawShape(const GraphicsContext*, const Shape&) const;
  void Clear(const GraphicsContext*, scaled, scaled, scaled, scaled) const;
  void Clear(const GraphicsContext*, scaled, scaled, const BoundingBox&) const;
  void Clear(const GraphicsContext*, const Rectangle&) const;
  void Clear(const GraphicsContext*, const Shape&) const;
  void Clear(const GraphicsContext*) const;

  void Update(scaled, scaled, scaled, scaled) const;
  void Update(const Rectangle&) const;
  void Update(void) const;

  const Rectangle* GetDrawingRect(void) const { return &drawingRect; }
  scaled GetWidth(void) const;
  scaled GetHeight(void) const;
  scaled GetTopX(void) const { return x0; }
  scaled GetTopY(void) const { return y0; }

protected:
  void ClearAux(const GraphicsContext*, scaled, scaled, scaled, scaled) const;

private:
  Rectangle  drawingRect;

  mutable scaled x;
  mutable scaled y;

  scaled x0;
  scaled y0;

  const GraphicsContext* selectedGC;

  // device dependent fields
  GtkWidget* gtk_widget;
  GdkPixmap* gdk_pixmap;
};

#endif
